package com.persagy.account.manage;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.collect.Lists;
import com.persagy.account.pojo.dto.SaasAccount;
import com.persagy.account.pojo.dto.SaasArea;
import com.persagy.account.pojo.dto.SaasGroup;
import com.persagy.account.pojo.dto.SaasProject;
import com.persagy.account.service.ISaasAccountService;
import com.persagy.account.service.ISaasGroupService;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.enums.ResponseCode;
import com.persagy.common.exception.BusinessException;
import com.persagy.common.utils.StringUtil;
import com.persagy.security.constant.CipherConstants;

import cn.hutool.core.util.BooleanUtil;
import cn.hutool.crypto.digest.MD5;

/**
 * @version 
 * @description 
 * @company persagy
 * @author zhangqiankun
 * @since 2021年3月18日:	上午10:20:22
 */
@Component
public class SaasAuthHandler {
	
	private static MD5 md5 = MD5.create();
	
    @Autowired
    private ISaasGroupService saasGroupService;

    @Autowired
    private ISaasAccountService saasAccountService;
    
    @Autowired
    private SaasAreaProjectHandler saasAreaProjectHandler;
    
    @Autowired
    private SaasGroupProjectHandler saasGroupProjectHandler;
    
	@Autowired
	private RedisTemplate<String, Object> redisTemplate;
    
    /**
     * 验证此账号的集团权限
     * 
     * @param accountId
     * @param groupCode
     * @return 
     * 	权限鉴定通过时: 
     *  有accountbelong 账号所属字段
     * 	若有返回集团编码，则此返回值必须作为代码向下执行的sql条件；
     * 	若有返回accountType，需进行 SaasCommonConstant.STR_STATUS_2.equals(accountType) 账号类型的判断
     */
    public Map<String, String> validAccountAuth(String accountId, String groupCode) {
    	// 获取此账号对应的集团权限
    	SaasAccount account = this.saasAccountService.getOne(accountId, null, null);
    	if (account == null) {
    		throw new BusinessException("账号信息为空");
		}
    	// 添加数据访问控制，账号所属，0-系统账号，1-业务账号
    	String accountBelong = account.getAccountBelong();
    	Map<String, String> result = new HashMap<String, String>(4);
		if (SaasCommonConstant.STR_STATUS_1.equals(accountBelong)) {
			String allowGroupCode = account.getGroupCode();
			if (StringUtil.isNotBlank(groupCode) && !groupCode.equals(allowGroupCode)) {
				throw new BusinessException("账号无该集团的访问权限");
			}
			groupCode = allowGroupCode;
		}
		result.put(CipherConstants.GROUP_CODE, groupCode);
		result.put(SaasCommonConstant.ACCOUNT_TYPE, account.getAccountType());
		result.put(SaasCommonConstant.ACCOUNT_BELONG, accountBelong);
		return result;
    }
    
    /**
     * 返回允许此账号访问的集团信息
     * 
     * @param accountId
     * @param groupCode
     * @return 允许此账号访问的集团信息
     */
    public List<SaasGroup> getAllowGroupList(String accountId, String groupCode) {
    	// 获取此账号对应的集团权限
    	Map<String, String> accountAuth = this.validAccountAuth(accountId, groupCode);
    	groupCode = accountAuth.get(CipherConstants.GROUP_CODE);
    	String accountType = accountAuth.get(SaasCommonConstant.ACCOUNT_TYPE);
		return this.getAllowGroupList(accountId, groupCode, accountType);
    }
    
    /**
     * 返回允许此账号访问的集团信息
     * 
     * @param accountId
     * @param groupCode
     * @param accountType
     * @return 允许此账号访问的集团信息
     */
    public List<SaasGroup> getAllowGroupList(String accountId, String groupCode, String accountType) {
    	List<SaasGroup> groups = null;
    	if (SaasCommonConstant.STR_STATUS_2.equals(accountType)) {
    		SaasGroup saasGroup = new SaasGroup();
			saasGroup.setGroupCode(groupCode);
			saasGroup.setValid(SaasCommonConstant.STATUS_1);
			groups = this.saasGroupService.queryAllowGroupInfo(saasGroup, accountId);
		} else {
			LambdaQueryWrapper<SaasGroup> queryWrapper = new SaasGroup.Builder().createQueryWrapper().groupCodeEq(groupCode).validEq(SaasCommonConstant.STATUS_1).builderQueryWrapper();
    		groups = this.saasGroupService.list(queryWrapper);
		}
		return groups == null ? Lists.newArrayList() : groups;
    }
    
    /**
     * 返回被允许访问的区域树
     * 
     * @param accountId
     * @param groupCode
     * @return 允许此账号访问的集团信息
     */
    public List<SaasArea> getAllowAreaList(String accountId, String groupCode, boolean excludeEmptyArea) {
    	// 获取此账号对应的集团权限
    	Map<String, String> accountAuth = this.validAccountAuth(accountId, groupCode);
    	groupCode = accountAuth.get(CipherConstants.GROUP_CODE);
    	String accountType = accountAuth.get(SaasCommonConstant.ACCOUNT_TYPE);
		return this.getAllowAreaList(accountId, groupCode, accountType, excludeEmptyArea);
    }
    
    /**
     * 返回被允许访问的区域树
     * 
     * @param accountId
     * @param groupCode
     * @param accountType
     * @return 
     */
    public List<SaasArea> getAllowAreaList(String accountId, String groupCode, String accountType, boolean excludeEmptyArea) {
    	List<SaasArea> topList = this.saasAreaProjectHandler.getTopAreaList(groupCode, excludeEmptyArea);
		if (SaasCommonConstant.STR_STATUS_2.equals(accountType)) {
			// 其他类型时，需要根据权限表数据，去匹配区域树
			this.saasAreaProjectHandler.queryAllowAreaTree(topList, accountId, groupCode);
		} else {
			this.saasAreaProjectHandler.querySaasAreaTree(topList, false);
		}
		return topList;
    }
    
    /**
     * 返回被允许访问的区域树
     * 
     * @param accountId
     * @param groupCode
     * @param areaId
     * @return 允许此账号访问的项目信息
     */
    public List<SaasProject> getAllowProjectList(String accountId, String groupCode, String areaId) {
    	// 获取此账号对应的集团权限
    	Map<String, String> accountAuth = this.validAccountAuth(accountId, groupCode);
    	groupCode = accountAuth.get(CipherConstants.GROUP_CODE);
    	String accountType = accountAuth.get(SaasCommonConstant.ACCOUNT_TYPE);
		return this.getAllowProjectList(accountId, groupCode, accountType, areaId);
    }
    
    /**
     * 返回被允许访问的区域树
     * 
     * @param accountId
     * @param groupCode
     * @param accountType
     * @param areaId
     * @return 
     */
    public List<SaasProject> getAllowProjectList(String accountId, String groupCode, String accountType, String areaId) {
    	boolean isAdmin = true;
    	if (SaasCommonConstant.STR_STATUS_2.equals(accountType)) {
    		isAdmin = false;
		}
    	List<SaasProject> projects = this.saasGroupProjectHandler.queryAllowProjectList(accountId, groupCode, areaId, isAdmin);
		return projects;
    }

    /**
     * 申请授权码，每次重新生成
     * 
     * @param clientId
     * @param redirectUrl
     * @param accountId
     * @param groupCode
     * @param appId
     * @return
     */
    public String getCode(String clientId, String redirectUrl, String accountId, String groupCode, String appId) {
    	// 验证客户端ID是否存在
    	Boolean member = this.redisTemplate.opsForSet().isMember(SaasCommonConstant.SAAS_CLIENT_ID_REDIS_KEY, clientId);
    	if (!BooleanUtil.isTrue(member)) {
    		throw new BusinessException(ResponseCode.A0001.getCode(), "非法客户端");
		}
    	// 如果redirectUrl存在，验证是否为可用值
    	if (!this.validRedirectUrl(clientId, redirectUrl)) {
    		throw new BusinessException(ResponseCode.A0001.getCode(), "非法客户端");
		}
    	// 设置账号相关信息
		this.redisTemplate.opsForHash().put(clientId, CipherConstants.APP_ID_HASH_KEY, appId);
		this.redisTemplate.opsForHash().put(clientId, CipherConstants.ACCOUNT_ID_HASH_KEY, accountId);
		this.redisTemplate.opsForHash().put(clientId, CipherConstants.GROUP_CODE_HASH_KEY, groupCode);
		
    	// 生成新的授权码，并验证授权码是否已存在
    	JSONObject temp = new JSONObject();
    	temp.put("time", System.currentTimeMillis());
    	temp.put("redirectUrl", redirectUrl);
    	temp.put("clientId", clientId);
    	String code = md5.digestHex(temp.toJSONString());
    	this.redisTemplate.opsForHash().put(clientId, CipherConstants.CODE_HASH_KEY, code);
    	return code;
    }
    
    /**
     * 验证redirectUrl的有效性，此方法默认空的redirectUrl，校验为true
     * 
     * @param clientId
     * @param redirectUrl
     * @return
     */
    public boolean validRedirectUrl(String clientId, String redirectUrl) {
    	if (StringUtil.isBlank(redirectUrl)) {
			return true;
		}
    	// 判断是否为内部客户端，内部不校验url
    	Boolean inner = (Boolean)this.redisTemplate.opsForHash().get(clientId, CipherConstants.INNER_CLIENT_HASH_KEY);
    	if (BooleanUtil.isTrue(inner)) {
    		return true;
		}
		Object object = this.redisTemplate.opsForHash().get(clientId, CipherConstants.REDIRECT_URL_HASH_KEY);
		if(object == null) {
			return false;
		}
		String allowUrl = (String) object;
		if (StringUtil.isBlank(allowUrl)) {
			return false;
		}
		if (allowUrl.equals(allowUrl)) {
			return true;
		}
    	
    	return false;
    }
    
}
